'==========================================================================
'
' NAME: GetPatchInfo.vbs
'
' AUTHOR: Guy Teverovsky , Microsoft
' DATE  : 4/8/2009
'
' COMMENT: 
'
'==========================================================================

Option Explicit

Const MSIINSTALLCONTEXT_NONE          = 0
Const MSIINSTALLCONTEXT_USERMANAGED   = 1
Const MSIINSTALLCONTEXT_USERUNMANAGED = 2
Const MSIINSTALLCONTEXT_MACHINE       = 4
Const MSIINSTALLCONTEXT_ALL           = 7
Const MSIINSTALLCONTEXT_ALLUSERMANAGED= 8

Const MSIPATCHSTATE_INVALID 	= 0
Const MSIPATCHSTATE_APPLIED 	= 1
Const MSIPATCHSTATE_SUPERSEDED 	= 2
Const MSIPATCHSTATE_OBSOLETED 	= 4
Const MSIPATCHSTATE_REGISTERED 	= 8
Const MSIPATCHSTATE_ALL 		= 15

Const MSISOURCETYPE_UNKNOWN = &H00000000
Const MSISOURCETYPE_NETWORK = &H00000001
Const MSISOURCETYPE_URL     = &H00000002
Const MSISOURCETYPE_MEDIA   = &H00000004
Const MSISOURCETYPE_ALL		= &H00000007

Const MSICODE_PRODUCT = &H00000000
const MSICODE_PATCH = 	&H40000000



Dim objInstaller
Dim objProducts, objProduct 
Dim objFeatures
Dim objPatches, objPatch
Dim objArgs
Dim strPatchGUID

Set objInstaller = CreateObject("WindowsInstaller.Installer") 
Set objPatches = objInstaller.PatchesEx("","", _
						MSIINSTALLCONTEXT_ALL, MSIPATCHSTATE_ALL, 0)


Select Case WScript.Arguments.Named.Count
	Case 0:	
		ShowAllPatchInfo
		
	Case 1:
		If WScript.Arguments.Named.Item("guid") <> "" Then
			ShowPatchInfoByGUID WScript.Arguments.Named.Item("guid")
		Else
			ShowHelp
		End If 
	
	Case Else: 
		ShowHelp
End Select 

Set objPatches 		= Nothing 
Set objInstaller 	= Nothing 


Function ShowAllPatchInfo
	For Each objPatch In objPatches
		DisplayPatchInfo objPatch
	Next 
End Function 

Function ShowPatchInfoByGUID(strGUID)
	Dim bFound : bFound = False 
	
	For Each objPatch In objPatches
		If Ucase(objPatch.PatchCode) = UCase(strGUID) Then
			DisplayPatchInfo objPatch
			bFound = True 
		End If 
	Next 	
	
	If Not bFound Then
		WScript.Echo "Failed to locate patch with GUID " & strGUID
	End If 
End Function 

Function DisplayPatchInfo(objPatch)
	WScript.Echo "-------------------------------------------------------"	
	WScript.Echo "Patch Name:    " & objPatch.PatchProperty("DisplayName")
	WScript.Echo "Patch Code:    " & objPatch.PatchCode	
	WScript.Echo "More Info URL: " & objPatch.PatchProperty("MoreInfoURL")
	WScript.Echo "Patch State:   " & DecodePatchState(objPatch.PatchProperty("State"))
	WScript.Echo "Product Code:  " & objPatch.ProductCode
	WScript.Echo "Product Name:  " & GetProductForPatch(objPatch)
End Function 

Function DecodePatchState(state)
	Select Case state
		Case 1:	DecodePatchState = "Installed"
		Case 2: DecodePatchState = "Superseded"
		Case 4: DecodePatchState = "Obsolete"
		Case Else: DecodePatchState = cstr(state)
	End Select 
End Function 

Function GetProductForPatch(objPatch)
	Set objProducts = objInstaller.ProductsEx("", "", MSIINSTALLCONTEXT_ALL)
	GetProductForPatch = "Not Found"
	For Each objProduct In objProducts
		If objProduct.ProductCode = objPatch.ProductCode Then 
			GetProductForPatch = objProduct.InstallProperty("InstalledProductName")
			Exit For 
		End If 
	Next 
End Function 

Function ShowHelp
	WScript.Echo "Displays details for installed patches/hotfixes"
	WScript.Echo  
	WScript.Echo "Usage: cscript " & WScript.ScriptName & " [/guid:<PATCHGUID>]"
	WScript.Echo 
	WScript.Echo "	/guid:<PATCHGUID>	The GUID of the hotfix to query"
	WScript.Echo 
	WScript.Echo "	Running the script without parameters will enumerate all"
	WScript.Echo "	the patches installed."
	
	WScript.Echo
End Function 